# bounding box europe
.f <- c(0, 0)
.bb <- c("xmin" = 2923408 - .f[1], "xmax" = 5309418 + .f[1],
         "ymin" = 1576769 - .f[2], "ymax" = 4855870 + .f[2])
.palette <- c("#2166ac", "#67a9cf", "#d1e5f0", "#EDE45F", "#fddbc7","#ef8a62", "#b2182b")

.min_range <- function(x, min = 3e5) {
  rng <- range(x)
  if(diff(rng) < min) {
    rng <- rng + c(-1, 1) * (min - diff(rng)) / 2
  }

  return(rng)
}


map <- function(x, limits = "data", f = 0.2) {

  require(ggplot2)
  if (length(f) == 1) f[2] <- f[1]

  limits <- if(is.list(limits) && length(limits) == 2) {
    limits
  } else if (limits == "europe"){
    list(x = .bb[c("xmin", "xmax")],
         y = .bb[c("ymin", "ymax")])
  } else {
    # fit data
    list(x = extendrange(x$x2, f = f[1]),
         y = extendrange(x$y2, f = f[2]))
  }

  limit <- lapply(limits, .min_range)

  ggplot2::ggplot(x, aes(x = x2, y = y2)) +
    ggplot2::geom_polygon(data = coastlines, aes(group = group),
                          col = "lightgrey", fill = "white", size = 0.1) +
    ggplot2::coord_fixed(xlim = limit[["x"]], ylim = limit[["y"]]) +
    theme_bw(10) +
    theme(axis.line=ggplot2::element_blank(),
          axis.text = ggplot2::element_blank(),
          axis.ticks = ggplot2::element_blank(),
          axis.title = ggplot2::element_blank(),
          panel.grid = ggplot2::element_blank(),
          plot.background=ggplot2::element_blank(),
          panel.background = element_rect(fill = "#C5DAE3") ,
          strip.background=ggplot2::element_rect(fill = "white"))
}

under_drought <- function(x, limits = c("data", "europe"), f = 0.2,
                          legend.position = "top", ...) {

  x <- melt_country(x)
  x <- subset(x, !duplicated(paste(x2, y2, sep = "-")))
  x$drought <- ifelse(x$unterminated, "still under drought", "no drought")
  x$drought <- factor(x$drought, levels = c("still under drought", "no drought"))

  map(x = x, limits = limits, f = f) +
    ggplot2::geom_point(data = x, ggplot2::aes(x2, y2, col = drought), size = size)  +
    theme(legend.position = legend.position)

}


compare_drought <- function(x, limits = c("data", "europe"), f = 0.2,
                           legend.position = "top", ncol = 3,
                           ...) {

  limits <- match.arg(limits, several.ok = FALSE)

  map(x = x, limits = limits, f = f) +
    geom_point(data = x, aes(x = x2, y = y2, colour = cl), ...) +
    ggplot2:: scale_colour_manual("return period \nin years", values = .palette) +
    facet_wrap(~period+month.name, ncol = ncol, drop = F,
               labeller = labeller(.multi_line = FALSE)) +
    theme(legend.position = legend.position)
}



plot_drought <- function(x, limits = c("data", "europe"), f = 0.2,
                        legend.position = "right", ...) {

  limits <- match.arg(limits, several.ok = FALSE)

  map(x = x, limits = limits, f = f) +
    ggplot2::geom_point(data = x, ggplot2::aes(x2, y2, color = cl), ...) +
    ggplot2::facet_grid(char~period) +
    ggplot2:: scale_colour_manual("return period \nin years", values = .palette) +
    theme(legend.position = legend.position)
}


plot_multiannual <- function(result, limits = c("data", "europe"), f = 0.2,
                             legend.position = "right", ...) {

  limits <- match.arg(limits, several.ok = FALSE)

  .extract_multiy <- function(x) {
    att <- attr(x, "lf")
    myear <- att$drought[names(att$drought) != "reference"]
    myear <- sapply(myear, function(x) x$multiyear)
    myear <- myear[sapply(myear, length) > 0]
    if(length(myear) == 0) return()

    myear <- data.frame(period = names(myear), duration = unlist(myear))

    gauge <- get_attr(x, drop.complex = T)

    cols <- c("x2", "y2", "station", "river", "region", "country")
    return(cbind(myear, gauge[cols]))
  }

  x <- do.call(rbind, lapply(result, .extract_multiy))
  #x <- subset(x, duration > 1)


  map(x = x, limits = limits, f = f) +
    ggplot2::geom_point(data = x, ggplot2::aes(x2, y2, size = as.factor(duration), col = as.factor(duration)), ...) +
    ggplot2::facet_wrap(~period, ncol = 1) +
    ggplot2::scale_size_manual("drought lasted n years", values = c("1" = 0.5, "2" = 1, "3" = 2, "4" = 2.5, "5" = 3)) +
    ggplot2::scale_color_manual("drought lasted n years", values = c("black", scales::hue_pal()(4))) +
    theme(legend.position = legend.position)
}
