.melt_matrix <- function(x) {
  y <- data.frame(period = rep(rownames(x), ncol(x)),
                  char = rep(colnames(x), each = nrow(x)),
                  value = as.vector(x), stringsAsFactors = FALSE)

  return(y)
}

# hack around Frances missing names
.extract_value <- function(x, key) {
  att <- attr(x, "lf")[key]
  names(att) <- key
  att[sapply(att, is.null)] <- NA

 return(att)
}

.extract_value_xts <- function(x, key) {
  att <- xtsAttributes(x)[key]
  names(att) <- key
  att[sapply(att, is.null)] <- NA

  return(att)
}


.melt_station <- function(x) {
  val <- x[c("return.period", "values")]

  country <- get_attr(x, drop.complex = F)["country"][[1]]
  cols <- c("station", "river", "country", "terminated", "unterminated", "catchment", "x2", "y2", "z")

  att <- get_attr(x, drop.complex = F)[cols]
  if(is.na(att$station)) att$station <- paste(att$coordinates[1:2], collapse = " - ")

  variable <- rep(names(val), times = sapply(val, length))
  y <- data.frame(variable, do.call(rbind, lapply(val, .melt_matrix)),
             att, row.names = NULL)

  y$cl <- classify_rp(y$value)


  return(y)
}

melt_country <- function(x) {
  y <- do.call(rbind.fill, lapply(x, .melt_station))
  y$char <- factor(y$char, levels = c("discharge", "duration", "volume",
                                    "intensity", "day"))

  symb <- droplevels(subset(y, variable == "values" & char == "day" &
                                 period == "reference"))
  symb <- subset(symb, !duplicated(paste(x2, y2, sep = "-")))

  rng <- as.numeric(format(as.Date(c("2015-05-01", "2015-12-01")), "%j"))

  symb$season <- ifelse(symb$value >= rng[1] & symb$value < rng[2],
                           "summer (May - Nov)", "winter")
  symb$season <- factor(symb$season, levels = c("summer (May - Nov)", "winter"))

  symb$drought <- ifelse(symb$unterminated, "still under drought", "no drought")
  symb$drought <- factor(symb$drought, levels = c("still under drought", "no drought"))


  y <- merge(y, symb[, c("x2", "y2", "season", "drought")], all.x = TRUE)

  # in germany, catchemnt is of type "character"
  y$catchment <- as.numeric(y$catchment)

  rownames(y) <- NULL
  return(y)
}


.melt_monthly <- function(x) {

  periods = attr(x, "lf")$periods

  att <- attr(x, "lf")[c("station", "river", "country", "unterminated", "coordinates")]
  if(is.na(att$station)) att$station <- paste(att$coordinates[1:2], collapse = " - ")
  att <- att[setdiff(names(att), "coordinates")]

  mm7 <- attr(x, "lf")$mm7
  mm7 <- lapply(periods, function(x) {
    tmp <- mm7[x, ]
    if(nrow(tmp) == 0) {
      data.frame(month = 1:12, discharge = NA)
    } else {
      data.frame(month = as.numeric(format(time(tmp), "%m")),
                 discharge = as.vector(tmp[, 1])) # should be tmp[, "discharge"]
    }})

  # mean of periods
  avg <- lapply(mm7, function(x) aggregate(x$discharge, x[, "month", drop = F], mean))

  df <- list()
  for(i in seq_along(avg)){
    df[[i]] <- data.frame(avg[[i]], period = names(avg)[i])
  }

  df <- do.call(rbind, df)
  df$month.name <- factor(month.name[df$month], levels = month.name)
  names(df)[names(df) == "x"] <- "discharge"
  df$return.period <- .rp_ev(x = df[, "discharge"], fit = x[["fit"]][["discharge"]])

  y <- data.frame(df, att, as.list(attr(x, "lf")$coordinates),
                  row.names = NULL)

  return(y)
}

melt_minima <- function(x) {
  y <- lapply(x, .melt_monthly)
  y <- do.call(rbind, y)

  rownames(y) <- NULL
  return(y)
}

