knit_with_arg <- function(input, outpath = ".", compiler = NULL,
                          envir = parent.frame(), quiet = TRUE,
                          encoding = "UTF-8", clean = TRUE, arg = list(), ...) {
  require(knitr)

  wd <- getwd()
  on.exit(setwd(wd))
  setwd(tempdir())

  file.copy(list.files(dirname(input), pattern = "\\.[Rr][Nn][Ww]$",
                       full.names = TRUE),
            tempdir(), overwrite = TRUE)

  outfile <- sub("_country_", arg[["country"]], basename(input))
  outfile <-  sub("\\..+", ".tex", basename(outfile))

  ss <- arg$subset[[1]]
  suffix <- names(arg$subset[1])
  is.subset <- is.numeric(ss) || is.logical(ss) && length(ss) > 1
  if(is.subset) outfile <- sub(".tex", paste0("_subset_", suffix, ".tex"), outfile)

  trimmed <- tryCatch(as.Date(arg$trimmed), error = function(x) NA)
  is.trimmed <- !is.na(trimmed)
  if(is.trimmed) outfile <- sub(".tex", "_trimmed.tex", outfile)

  output <- file.path(getwd(), outpath, outfile)


  env <- new.env()
  for(i in seq_along(arg)){
    assign(names(arg)[i], value = arg[[i]], envir = env)
  }

  knit2pdf(input = file.path(tempdir(), basename(input)),
           output = outfile,
           envir = env, clean = clean, quiet = quiet,
           encoding = encoding)

  name <- sub("tex$", "", outfile)
  res <- list.files(tempdir(),  pattern = paste0(name, "pdf"),
                    full.names = TRUE)

  setwd(wd)
  a <- file.copy(from = res, to = file.path(outpath, basename(res)),
                 overwrite = TRUE)

  message("files copied to:\n", paste(file.path(outpath, basename(res)), collapse = "\n"))

}



country_report <- function(country,
                           f = list(season = 0.04, annual = 0.04, monthly = 0.04),
                           lp = list(season = "top", annual = "top", monthly = "top"),
                           fig.height = list(coverage = 25.2/2.54),
                           result.path = paste0("M:/papers/hpt/analysis/", country),
                           outpath = ".", subset = NULL) {
  if(is.null(subset)) subset <-  TRUE

  knit_with_arg(input = system.file("_country_-country_report.Rnw", package = "drought2015"),
                outpath = outpath,
                arg = list(country = country, f = f, lp = lp,
                           fig.height = fig.height, result.path = result.path,
                           subset = subset, trimmed = as.Date("2015-10-31")))

  knit_with_arg(input = system.file("_country_-country_report.Rnw", package = "drought2015"),
                outpath = outpath,
                arg = list(country = country, f = f, lp = lp,
                           fig.height = fig.height, result.path = result.path,
                           subset = subset, trimmed = FALSE))
}


station_report <- function(country,
                           result.path = paste0("M:/papers/hpt/analysis/", country),
                           outpath = ".", subset = NULL) {
  knit_with_arg(input = system.file("_country_-station_report.Rnw", package = "drought2015"),
                outpath = outpath,
                arg = list(country = country, result.path = result.path,
                           subset = subset, trimmed = as.Date("2015-10-31")))

  knit_with_arg(input = system.file("_country_-station_report.Rnw", package = "drought2015"),
                outpath = outpath,
                arg = list(country = country, result.path = result.path,
                           subset = subset, trimmed = FALSE))

}


europe_report <- function(country,
                           f = list(season = 0.04, annual = 0.04, monthly = 0.04),
                           lp = list(season = "top", annual = "top", monthly = "top"),
                           fig.height = list(coverage = 25.2/2.54),
                           result.path = "M:/papers/hpt/europe/",
                           outpath = ".", subset = NULL) {
  if(is.null(subset)) subset <-  TRUE

  knit_with_arg(input = system.file("europe_report.Rnw", package = "drought2015"),
                outpath = outpath,
                arg = list(country = "europe", result.path = result.path,
                           trimmed = as.Date("2015-10-31")))

  knit_with_arg(input = system.file("europe_report.Rnw", package = "drought2015"),
                outpath = outpath,
                arg = list(country = "europe", result.path = result.path,
                           trimmed = FALSE))
}
