\name{import}
\alias{import}

\title{Imports and preprocesses discharge Files}
\description{
Reads the contents from a csv file and appends some meta information.
}
\usage{
import(file, fun = read.csv, station.info, format = "\%Y-\%m-\%d", ...)
}

\arguments{
  \item{file}{A character vector of length one containing the name of the file to be imported. The first column of the file is assumed to contain the timestamp as described by the argument \code{format}. The second column contains discharges.}
  \item{fun}{the function used for reading the csv file. Usually eihter \code{\link{read.csv}} or \code{\link{read.csv2}}.}
  \item{station.info}{A dataframe with station metadata. It should at least contain the columns: external id (eid), coordinates (x, y, z), region, catchment and filename.}
  \item{format}{format string of the time column.}
  \item{\dots}{other arguments, passed on to \code{fun} which is used for import.}
}
\value{Returns an xts object containing a regular time series of daily discharges. Gaps in the input file are padded with NA values.}


\examples{
# path of austrian sample files
path <- paste0(system.file(package="drought2015"), "/extdata/at")

# list all files in subdirectory "discharge"
files <- list.files(file.path(path, "discharge"),
                    pattern = "*\\\\.csv$", full.names = TRUE)

# read meta data which is not in the header of the nrfa files
gauge <- read.csv(file.path(path, "gauge.csv"), as.is = TRUE,
                  fileEncoding = "UTF-8")
gauge

# coordinate transformation
xy <- transform_crs(gauge[, c("x", "y")], from = CRS("+init=epsg:31287"))
gauge <- cbind(gauge, xy)

# import all observed discharges
# the content of every files is an element of a list
infile <- lapply(files, import, gauge = gauge)

# do the comparison for all streamflow gauges
results <- lapply(infile, compare_lf)

# print summary and extreme value plots
summary(results[[1]])
plot(results[[1]])
}
